<?php

namespace FiloBlu\DiscountRule\Plugin\Model\Rule\Condition;

use Magento\Framework\Model\AbstractModel;
use Magento\Quote\Api\Data\TotalsInterface;
use Magento\SalesRule\Model\Rule\Condition\Address as SalesRuleAddress;

/**
 * Class Address
 * @package FiloBlu\ExtCartRule\Plugin\Model\Rule\Condition
 */
class Address
{
    const KEY_SUBTOTAL_WITH_DISCOUNT_NO_SHIPPING = 'subtotal_with_discount_exclude_shipping';
    const KEY_SUBTOTAL_WITH_GIFTCARD_NO_SHIPPING = 'subtotal_with_giftcard_exclude_shipping';
    const KEY_SUBTOTAL_WITH_TAX_NEW = 'base_subtotal_total_incl_tax';

    /**
     * Load attribute options
     *
     * @param $subject SalesRuleAddress
     * @param $result SalesRuleAddress
     * @return SalesRuleAddress
     */
    public function afterLoadAttributeOptions($subject, $result)
    {
        $attributes = $subject->getData('attribute_option');
        if(!isset($attributes['payment_method'])){
            $attributes['payment_method'] = __('Payment Method');
        }
        $attributes[TotalsInterface::KEY_BASE_SUBTOTAL_INCL_TAX] = __('Subtotal (Including Tax)');
        $attributes[TotalsInterface::KEY_BASE_GRAND_TOTAL] = __('Base Grand Total');
        $attributes[TotalsInterface::KEY_SUBTOTAL_WITH_DISCOUNT] = __('Subtotal With Discount');
        $attributes[TotalsInterface::KEY_BASE_SUBTOTAL_WITH_DISCOUNT] = __('Base Subtotal With Discount');
        $attributes[TotalsInterface::KEY_GRAND_TOTAL] = __('Grand Total');
        $attributes[self::KEY_SUBTOTAL_WITH_DISCOUNT_NO_SHIPPING] = __('Subtotal with discounts no shipping (Incl. Tax)');
        $attributes[self::KEY_SUBTOTAL_WITH_GIFTCARD_NO_SHIPPING] = __('Subtotal with giftcard no shipping (Incl. Tax)');
        $attributes[self::KEY_SUBTOTAL_WITH_TAX_NEW] = __('Subtotal (Incl. Tax) New');
        $subject->setData('attribute_option', $attributes);
        return $result;
    }

    /**
     * Get input type
     *
     * @param $subject SalesRuleAddress
     * @param $result string
     * @return string
     */
    public function afterGetInputType($subject, $result)
    {
        switch ($subject->getData('attribute')) {
            case TotalsInterface::KEY_BASE_SUBTOTAL_INCL_TAX:
            case TotalsInterface::KEY_BASE_GRAND_TOTAL:
            case TotalsInterface::KEY_SUBTOTAL_WITH_DISCOUNT:
            case TotalsInterface::KEY_BASE_SUBTOTAL_WITH_DISCOUNT:
            case TotalsInterface::KEY_GRAND_TOTAL:
            case self::KEY_SUBTOTAL_WITH_DISCOUNT_NO_SHIPPING:
            case self::KEY_SUBTOTAL_WITH_GIFTCARD_NO_SHIPPING:
            case self::KEY_SUBTOTAL_WITH_TAX_NEW:
                return 'numeric';

        }
        return $result;
    }

    /**
     * @param $subject
     * @param AbstractModel $model
     * @return AbstractModel[]|null
     */
    public function beforeValidate($subject, AbstractModel $model)
    {
        if ((int)$model->getGiftCardsAmount()) {
            /* check if base grand total is less or equal than rule value */
            if (($model->getBaseGrandTotal() <= $subject->getValue())) {
                /* then change attribute name to use for validating checks */
                $subject->setData('attribute', 'base_subtotal_total_incl_tax');
            }
            return null;
        }

        if (self::KEY_SUBTOTAL_WITH_DISCOUNT_NO_SHIPPING !== $subject->getAttribute()) {
            return  null;
        }

        $model->setData(self::KEY_SUBTOTAL_WITH_DISCOUNT_NO_SHIPPING, $model->getSubtotalInclTax() + $model->getDiscountAmount());

        return [$model];
    }
}
